<?php
namespace CRTPCreativeProducts;

use CRTPCreativeProducts\PageSettings\Page_Settings;
define( "CRTP_ASFSK_ASSETS_ADMIN_DIR_FILE", plugin_dir_url( __FILE__ ) . "assets/admin" );
class CRTPCreativeClassProducts {

	private static $_instance = null;

	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	public function crtp_admin_editor_scripts() {
		add_filter( 'script_loader_tag', [ $this, 'crtp_admin_editor_scripts_as_a_module' ], 10, 2 );
	}

	public function crtp_admin_editor_scripts_as_a_module( $tag, $handle ) {
		if ( 'crtp_the_service_editor' === $handle ) {
			$tag = str_replace( '<script', '<script type="module"', $tag );
		}
		return $tag;
	}

	private function include_widgets_files() {
		require_once( __DIR__ . '/widgets/crtp-products.php' );
	}

	public function crtp_register_widgets() {
		// Its is now safe to include Widgets files
		$this->include_widgets_files();

		// Register Widgets
		\Elementor\Plugin::instance()->widgets_manager->register_widget_type( new Widgets\CRTPProducts() );
	}

	private function add_page_settings_controls() {
		require_once( __DIR__ . '/page-settings/super-products-manager.php' );
		new Page_Settings();
	}

	// Register Category
	public function crtp_add_elementor_widget_categories( $elements_manager ) {

		$elements_manager->add_category(
			'bwdcrpt_creative_products_category',
			[
				'title' => esc_html__( 'Creative Products Grid', 'creative-products' ),
				'icon' => 'eicon-person',
			]
		);
	}
	public function crtp_all_assets_for_the_public(){
		wp_enqueue_style( 'crtp-bootstrap', 'https://cdnjs.cloudflare.com/ajax/libs/bootstrap/5.2.0/css/bootstrap.min.css');
		wp_enqueue_style( 'crtp-main-style', 'https://bwd-elementor-addons-pro.netlify.app/crtp_css.css');
	}
	public function crtp_all_assets_for_elementor_editor_admin(){
		$all_css_js_file = array(
            'crtp_products_admin_icon_css' => array('crtp_path_admin_define'=>CRTP_ASFSK_ASSETS_ADMIN_DIR_FILE . '/icon.css'),
        );
        foreach($all_css_js_file as $handle => $fileinfo){
            wp_enqueue_style( $handle, $fileinfo['crtp_path_admin_define'], null, '2.1', 'all');
        }
	}

	public function __construct() {
		// For public assets
		add_action('wp_enqueue_scripts', [$this, 'crtp_all_assets_for_the_public']);

		// For Elementor Editor
		add_action('elementor/editor/before_enqueue_scripts', [$this, 'crtp_all_assets_for_elementor_editor_admin']);
		
		// Register Category
		add_action( 'elementor/elements/categories_registered', [ $this, 'crtp_add_elementor_widget_categories' ] );

		// Register widgets
		add_action( 'elementor/widgets/widgets_registered', [ $this, 'crtp_register_widgets' ] );

		// Register editor scripts
		add_action( 'elementor/editor/after_enqueue_scripts', [ $this, 'crtp_admin_editor_scripts' ] );
	}
}

// Instantiate Plugin Class
CRTPCreativeClassProducts::instance();